//  Copyright (c) 1999-2008  Pavel Rychly

#ifndef CORPCONF_HH
#define CORPCONF_HH

#include <map>
#include <vector>
#include <string>
#include <exception>

class CorpInfo {
public:
    typedef enum {Unset_type, Corpus_type, Attr_type, Struct_type, 
		  Proc_type} type_t;
    type_t type;
    typedef std::map<std::string,std::string> MSS;
    typedef std::vector<std::pair<std::string,CorpInfo*> > VSC;
protected:
    CorpInfo* find_sub (const std::string &val, VSC &v);
public:
    MSS opts;
    VSC attrs;
    VSC structs;
    VSC procs;

    CorpInfo (type_t t=Unset_type);
    CorpInfo (CorpInfo *x);
    ~CorpInfo ();
    std::string dump (int indent=0);
    void set_defaults(type_t d_type=Corpus_type);
    MSS& find_attr (const std::string &attr);
    CorpInfo* find_struct (const std::string &attr)
        {return find_sub (attr, structs);}

    std::string &find_opt (const std::string &path);
    static bool str2bool (const std::string &str);

};


CorpInfo *loadCorpInfo (const std::string &corp_name_or_path);

class CorpInfoNotFound: public std::exception {
    const std::string _what;
public:
    const std::string name;
    CorpInfoNotFound (const std::string &name)
	:_what ("CorpInfoNotFound (" + name + ")"), name (name) {}
    virtual const char* what () const throw () {return _what.c_str();}
    virtual ~CorpInfoNotFound() throw() {}
};


#endif
